//Modified Bloom from Robobo1221
//https://www.shadertoy.com/view/lsBfRc

const float lods[7] = float[7](2.0,
                               3.0,
                               4.0,
                               5.0,
                               6.0,
                               7.0,
                               8.0);

const vec2 offsets[7] = vec2[7](vec2(0.0, 0.0),
                                vec2(0.5, 0.0),
                                vec2(0.0, 0.5),
                                vec2(0.125, 0.5),
                                vec2(0.175, 0.5),
                                vec2(0.225, 0.5),
                                vec2(0.275, 0.5));

#if defined composite2

vec3 renderTiles(const float lod, vec2 offset){
  const int steps = 5 - 1;
  const float rSteps = 1.0 / (steps + 1);

  float lodScale = exp2(lod);

  vec2 scale = rPixel * lodScale;
  vec2 sampleCoord = (coord - offset) * lodScale;

  if (any(greaterThanEqual(abs(sampleCoord - 0.5), scale + 0.5))) return vec3(0.0);

  vec3 bloom = vec3(0.0);
  float totalWeight = 0.0;

  for(int i = -steps; i <= steps; i++){
    for(int j = -steps; j <= steps; j++){
      float sampleLength = 1.0 - length(vec2(i, j)) * rSteps;
      float sampleWeight = clamp01(pow6(sampleLength));

      vec2 sampleOffset = sampleCoord + vec2(i - 0.98, j - 0.95) * scale + lodScale * rPixel;

      bloom += toGamma(texture2DLod(colortex5, sampleOffset, lod).rgb) * sampleWeight;
      totalWeight += sampleWeight;
    }
  }
  return bloom / totalWeight;
}

vec4 renderBloom(){
  vec3 bloom = vec3(0.0);
  #ifdef BLOOM
  for(int i = 0; i < 7; ++i){
    bloom += renderTiles(lods[i], offsets[i]);
  }
  #endif

  return EncodeRGBE8(max0(bloom));
}

#elif defined composite3

vec3 renderBloomTile(vec2 coord, const float lod, vec2 offset){
	float lodScale = exp2(-lod);

	vec2 sampleCoord = (coord * lodScale) + offset;

  return max0(toLinear(DecodeRGBE8(texture2DLod(colortex3, sampleCoord, lod))));
}

vec3 renderBloom(vec2 coord, float avg){
	vec3 bloom = vec3(0.0);
  float totalWeight = 0.0;

	#ifdef BLOOM
  for(int i = 0; i < 7; ++i){
    float sampleWeight = 1.0 / (lods[i] + 1.0);
    bloom += renderBloomTile(coord, lods[i], offsets[i]) * sampleWeight;
    totalWeight += sampleWeight;
  }
	#endif

	float bloomAVG = avg + BLOOM_STRENGTH;

	return DecodeColor(bloom / totalWeight) * exp2(bloomAVG - 3.25) + isEyeInWater * 3.0;
}

#endif
