//Temporal Anti Aliasing from Pissang
//https://github.com/pissang/claygl-advanced-renderer/blob/master/src/TAA.glsl

vec3 renderCurrentColor(vec2 coord){
  return toGamma(texture2D(colortex0, coord).rgb);
}

vec3 renderPreviousColor(vec2 coord){
  return toGamma(texture2D(colortex4, coord).rgb);
}

vec2 renderVelocity(vec3 screenPos) {
  vec3 projection = renderViewPosition(screenPos);
		   projection = renderWorldPosition(projection);
       projection = (cameraPosition - previousCameraPosition) + projection;
		   projection = transMAD(gbufferPreviousModelView, projection);
       projection = (diagonal4(gbufferPreviousProjection).xyw * projection + gbufferPreviousProjection[3].w) / -projection.z * 0.5 + 0.5;

  return screenPos.xy - projection.xy;
}

vec3 renderClosestFragment(sampler2D depth, vec2 uv, vec2 pixelSize) {
	vec2 dd = abs(pixelSize);
	vec2 du = vec2(dd.x, 0.0);
	vec2 dv = vec2(0.0, dd.y);

	vec3 dtl = vec3(-1, -1, texture2D(depth, uv - dv - du).x);
	vec3 dtc = vec3( 0, -1, texture2D(depth, uv - dv).x);
	vec3 dtr = vec3( 1, -1, texture2D(depth, uv - dv + du).x);

	vec3 dml = vec3(-1, 0, texture2D(depth, uv - du).x);
	vec3 dmc = vec3( 0, 0, texture2D(depth, uv).x);
	vec3 dmr = vec3( 1, 0, texture2D(depth, uv + du).x);

	vec3 dbl = vec3(-1, 1, texture2D(depth, uv + dv - du).x);
	vec3 dbc = vec3( 0, 1, texture2D(depth, uv + dv).x);
	vec3 dbr = vec3( 1, 1, texture2D(depth, uv + dv + du).x);

	vec3 dmin = dtl;
       dmin = dmin.z > dtc.z ? dtc : dmin;
       dmin = dmin.z > dtr.z ? dtr : dmin;

       dmin = dmin.z > dml.z ? dml : dmin;
       dmin = dmin.z > dmc.z ? dmc : dmin;
       dmin = dmin.z > dmr.z ? dmr : dmin;

       dmin = dmin.z > dbl.z ? dbc : dmin;
       dmin = dmin.z > dbc.z ? dbc : dmin;
       dmin = dmin.z > dbc.z ? dbc : dmin;

	return vec3(uv + dd.xy * dmin.xy, dmin.z);
}

#if defined composite4
vec3 clipAABB(vec3 aabbMin, vec3 aabbMax, vec3 p, vec3 q) {
	vec3 p_clip = 0.5 * (aabbMax + aabbMin);
	vec3 e_clip = 0.5 * (aabbMax - aabbMin) + 1e-8;

	vec3 v_clip = q - p_clip;
	vec3 v_unit = v_clip.xyz / e_clip;
	vec3 a_unit = abs(v_unit);
	float ma_unit = max(a_unit.x, max(a_unit.y, a_unit.z));

  return ma_unit > 1.0 ? p_clip + v_clip / ma_unit : q;
}

vec3 renderTemporalProjection(vec2 coord, vec2 velocity){
  vec3 current = renderCurrentColor(coord);
  vec3 previous = renderPreviousColor(coord - velocity);

  vec2 dx = vec2(rPixel.x, 0.0);
  vec2 dy = vec2(0.0, rPixel.y);

  vec3 ctl = renderCurrentColor(coord - dy - dx);
  vec3 ctc = renderCurrentColor(coord - dy);
  vec3 ctr = renderCurrentColor(coord - dy + dx);

  vec3 cml = renderCurrentColor(coord - dx);
  vec3 cmc = renderCurrentColor(coord);
  vec3 cmr = renderCurrentColor(coord + dx);

  vec3 cbl = renderCurrentColor(coord + dy - dx);
  vec3 cbc = renderCurrentColor(coord + dy);
  vec3 cbr = renderCurrentColor(coord + dy + dx);

  vec3 cmin = min(ctl, min(ctc, min(ctr, min(cml, min(cmc, min(cmr, min(cbl, min(cbc, cbr))))))));
  vec3 cmax = max(ctl, max(ctc, max(ctr, max(cml, max(cmc, max(cmr, max(cbl, max(cbc, cbr))))))));

  vec3 cmin5 = min(ctc, min(cml, min(cmc, min(cmr, cbc))));
  vec3 cmax5 = max(ctc, max(cml, max(cmc, max(cmr, cbc))));

  vec3 cavg = (ctl + ctc + ctr + cml + cmc + cmr + cbl + cbc + cbr) / 9.0;
  vec3 cavg5 = (ctc + cml + cmc + cmr + cbc) / 5.0;

  cmin = 0.5 * (cmin + cmin5);
  cmax = 0.5 * (cmax + cmax5);
  cavg = 0.5 * (cavg + cavg5);

  previous = clipAABB(cmin, cmax, clamp(cavg, cmin, cmax), previous);

  float currentLum = renderLuminance701(current);
  float previousLum = renderLuminance701(previous);

  coord -= velocity;

  float unbias = abs(currentLum - previousLum) / max(currentLum, max(previousLum, 0.2));
  float weight = mix(TEMPORAL_WEIGHT, 0.97, pow6(1.0 - unbias));
        weight = mix(weight, 0.0, float(clamp01(coord) != coord));

  current += (1.0 - exp2(-(current - clamp(cavg, cmin, cmax)))) * TEMPORAL_SHARPERNING;
  current = mix(current, cavg, vec3(TEMPORAL_SOFTNESS));

  return mix(current, previous, weight);
}

vec3 renderTemporalAA(vec2 coord){
  vec3 closest = renderClosestFragment(depthtex0, coord, rPixel);
  vec2 velocity = renderVelocity(closest);

  #ifdef TEMPORAL_AA
  vec3 temporal = renderTemporalProjection(coord, velocity);
  #else
  vec3 temporal = renderCurrentColor(coord);
  #endif

  return temporal;
}
#elif defined composite5
vec3 renderMotionBlur(vec2 coord){
  #ifdef MOTION_BLUR
  vec3 closest = renderClosestFragment(depthtex0, coord, rPixel);
  vec2 velocity = renderVelocity(closest) * 0.1 / frameTime * 0.075 * MOTION_BLUR_STRENGTH;
       velocity = clamp(velocity, vec2(-0.045), vec2(0.045));
       velocity = closest.z < 0.7 ? vec2(0.0) : velocity;

  float dither = bayer64(gl_FragCoord.xy);

  vec3 color = vec3(0.0);
  float totalWeight = 0.0;

  for(int i = -2; i < 2; ++i){
    vec2 sampleOffset = coord + velocity * (float(i + dither) / 2.0);

    color += texture2D(colortex4, sampleOffset).rgb;
    totalWeight += 1.0;
  }

  return color / totalWeight;
  #else
  return texture2D(colortex4, coord).rgb;
  #endif
}
#endif
