const float fExposureBias = TONEMAP_EXPOSURE;
#if TONEMAP_FILMIC_PRESET == 0
const float fShoulderStrength = 0.15;
const float fLinearStrength = 0.50;
const float fLinearAngle = 0.10;
const float fToeStrength = 0.20;
const float fToeNumerator = 0.020;
const float fToeDenominator = 0.30;
const float fLinearWhite = 11.2;
#elif TONEMAP_FILMIC_PRESET == 1
const float fShoulderStrength = 0.28;
const float fLinearStrength = 0.29;
const float fLinearAngle = 0.10;
const float fToeStrength = 0.20;
const float fToeNumerator = 0.025;
const float fToeDenominator = 0.35;
const float fLinearWhite = 15.3;
#else
const float fShoulderStrength = 0.15 * TONEMAP_SHOULDER_S;
const float fLinearStrength = 0.50 * TONEMAP_LINEAR_S;
const float fLinearAngle = 0.10 * TONEMAP_LINEAR_A;
const float fToeStrength = 0.20 * TONEMAP_TOE_S;
const float fToeNumerator = 0.020 * TONEMAP_TOE_N;
const float fToeDenominator = 0.30 * TONEMAP_TOE_D;
const float fLinearWhite = 11.2 * TONEMAP_LINEAR_W;
#endif

vec3 filmic(vec3 color){
	return ((color * (fShoulderStrength * color + fLinearStrength * fLinearAngle) + fToeStrength * fToeNumerator) / (color * (fShoulderStrength * color + fLinearStrength) + fToeStrength * fToeDenominator)) - fToeNumerator / fToeDenominator;
}

vec3 FilmicTonemap(vec3 color){

	const vec3 linearWhite = vec3(fLinearWhite);

	vec3 whiteScale = filmic(linearWhite);
	vec3 filmic = filmic(color * fExposureBias);

	color = filmic * whiteScale;
	#if TONEMAP_FILMIC_PRESET == 0 || TONEMAP_FILMIC_PRESET == 2
	color *= 2.0;
	#endif

	return color;
}

vec3 BOTWTonemap(vec3 color){
	color = pow(color, vec3(5.0));
	color = color / (1.0 + color / TONEMAP_EXPOSURE);
	color = pow(color, vec3(1.0 / 5.0));

	float avg = (color.r + color.g + color.b) / 3.0;
	float maxc = max(color.r, max(color.g, color.b));

	float w = 1.0 - pow2(1.0 - 2.0 * avg);
	float weight = 1.0 + w * 0.35;

	return mix(vec3(maxc), color, weight);
}

vec3 Contrasty(vec3 color){
	color *= 0.4;
	color = BOTWTonemap(color);
	color = clamp01(TONEMAP_EXPOSURE * color);

	float max_color = max(color.r, max(color.g, color.b));
	float min_color = min(color.r, min(color.g, color.b));

	float color_saturation = max_color - min_color;
	float saturation = 0.2;

	float luma = renderLuminance701(color);

	color = mix(vec3(luma), color, 1.0 + (saturation * (1.0 - fsign(saturation) * color_saturation)));

	return color;
}

vec3 basicReinhardTonemap(vec3 color){
	color *= 0.55;
	return color / (color + 1.0);
}

vec3 renderTonemap(vec3 color){
	#if TONEMAP_MODE == 0
	color = basicReinhardTonemap(color);
	#elif TONEMAP_MODE == 1
	color = FilmicTonemap(color);
	#else
	color = Contrasty(color);
	#endif

	return toLinear(color);
}
