#if defined vertex

out vec2 coord;

void main(){
  coord = gl_Vertex.xy;
  gl_Position = vec4(gl_Vertex.xy * 2.0 - 1.0, 0.0, 1.0);
}
#elif defined fragment

in vec2 coord;

vec3 renderWidescreen(vec3 color){
  float up = 0.9 - WIDE_SCREEN_SIZE;
  float down = 0.1 + WIDE_SCREEN_SIZE;

  #ifdef WIDE_SCREEN
  if(coord.t > up || coord.t < down) color = vec3(0.0);
  #endif

  return color;
}

vec3 renderVignette(vec3 color){
  #ifndef VIGNETTE
  return color;
  #endif

  float dist = pow(distance(coord, vec2(0.5)), 3.5) * 1.25;

  return color * (1.0 - dist);
}

vec3 renderSaturation(vec3 color){
  vec3 saturation = vec3(renderLuminance601(color));
  return mix(saturation, color, TONEMAP_SATURATION);
}

vec3 renderContrast(vec3 color){
  float contrast = length(color);

  vec3 rContrast = color / contrast;

  contrast = pow(contrast, TONEMAP_CONTRAST);

  return rContrast * contrast;
}

vec3 renderWhiteClip(vec3 color){
  return clamp01(color * (0.1 + TONEMAP_WHITE_CLIP));
}

void main(){
  finalColor = toLinear(texture2D(colortex4, coord).rgb);

  finalColor = renderSaturation(finalColor);
  finalColor = renderContrast(finalColor);
  finalColor = renderWhiteClip(finalColor);

  finalColor = renderVignette(finalColor);
  finalColor = clamp01(finalColor);
  finalColor = toGamma(finalColor);

  finalColor = renderWidescreen(finalColor);

  finalColor += bayer64(gl_FragCoord.xy) / 64.0;
}
#endif
