float GerstnerWave(vec2 coord, vec2 waveDirection, float globalTime, float waveSteepness, float waveAmplitude, float waveLength) {
  const float g = 9.81;

  // Wave number calculation
  float k = TAU / waveLength;
  float w = sqrt(g * k);

  // Count waves
  float x = k * dot(waveDirection, coord) - w * globalTime;
  float wave = sin(x) * 0.5 + 0.5;

  return waveAmplitude * pow(wave, waveSteepness);
}

float renderWaves(vec2 coord){
  float waveLength = 5.5;
  float waveAmplitude = 0.07;
  float waveSteepness = 0.7;

  float time = TIME * 0.4;
		    //time *= mix(1.0, 0.0, isIce);

  vec2 waveDirection = vec2(0.7, 0.4);

  vec2 anpos = coord * 0.025;

  float waves = 0.0;

  const float r = 0.25;
  const vec2 sc = vec2(sin(r), cos(r));
  const mat2 rot = mat2(sc.y, -sc.x, sc.x, sc.y);

  for (int i = 0; i < WATER_QUALITY; ++i){
    vec2 addNoise = render2DNoise(anpos * inversesqrt(waveLength)).xy * 2.0 - 1.0;

    waves += GerstnerWave(coord + (addNoise) * sqrt(waveLength), waveDirection, time, waveSteepness, waveAmplitude, waveLength);

    waveLength *= 0.7;
    waveAmplitude *= 0.6;
    waveSteepness *= 0.95;

    waveDirection *= rot;
    anpos *= rot;
    time *= 1.075;
  }
  return -waves;
}

#if defined water
vec2 renderParallaxWater(vec3 position, vec3 tangentVec){
  const int steps = 5;
  const float rSteps = inversesqrt(steps);

  const float maxHeight = WAVE_HEIGHT;

  vec3 increment = rSteps * tangentVec / -tangentVec.z;

  float height = renderWaves(position.xz);

  vec3 offset = -height * increment;
        height = renderWaves(position.xz + offset.xy) * maxHeight;

  for (int i = 1; i < steps - 1 && height < offset.z; ++i) {
    offset = (offset.z - height) * increment + offset;
    height = renderWaves(position.xz + offset.xy) * maxHeight;
  }

  if (steps > 1) {
   offset.xy = (offset.z - height) * increment.xz + offset.xy;
  }

  position.xz += offset.xy;

  return position.xz;
}
#endif

vec3 renderWaveNormal(vec3 coord){
    const float delta = 0.06;

    vec2 waves;
         waves.x = renderWaves(coord.xz + vec2(delta, -delta));
         waves.y = renderWaves(coord.xz + vec2(-delta, delta));
         waves -= renderWaves(coord.xz - vec2(delta));

    vec3 normal = vec3(-2.0 * delta, -2.0 * (delta * delta + delta), 4.0 * delta * delta);
         normal.xy *= waves;

    return normal;
}
