#if defined vertex

out vec2 coord;
out vec2 lmcoord;

out vec3 position;
out vec3 worldPosition;
out vec3 normal;

out vec4 color;

flat out float material;

flat out mat3 tbn;

#include "/program/gbuffers/common/curve.vsh"
#include "/program/gbuffers/common/displacement.vsh"
#include "/program/gbuffers/common/tbn.vsh"

void main(){

  coord = gl_MultiTexCoord0.xy;
  lmcoord = gl_MultiTexCoord1.xy / 255.0;

  position = transMAD(gl_ModelViewMatrix, gl_Vertex.xyz);
  worldPosition = renderVertexDisplacement(transMAD(shadowModelViewInverse, position));
  worldPosition = renderEarthCurvature(worldPosition);
  position = transMAD(shadowModelView, worldPosition);
  normal = (gl_NormalMatrix * gl_Normal);

  if(blockEntityId == 1019 || normal.z < 0.0){
    gl_Position = vec4(-1.0);
    return;
  }

  color = gl_Color;

  material = mc_Entity.x;

  tbn = renderTBN(normal);

  gl_Position = position.xyzz * diagonal4(gl_ProjectionMatrix) + gl_ProjectionMatrix[3];
  gl_Position.xyz = renderShadowDistortPosition(gl_Position.xyz);
}
#elif defined fragment
layout(location = 0) out vec4 color0;
layout(location = 1) out vec4 color1;

in vec2 coord;
in vec2 lmcoord;

in vec3 position;
in vec3 worldPosition;
in vec3 normal;

in vec4 color;

flat in float material;

flat in mat3 tbn;

bool isWater = material == 1006;
bool isIce = material == 1007;
bool isStainedGlass = material == 1008;

#include "/program/gbuffers/common/waterNormal.glsl"

vec4 renderAlbedo(){
  vec4 albedo = texture2D(texture, coord) * color;
  if(albedo.a == 0.0) discard;
	     albedo = isWater ? vec4(1.0) : albedo;

  return albedo;
}

void main(){
  if(!gl_FrontFacing) discard;
  
  color0 = renderAlbedo();

  vec3 shadowNormal = normal;
       shadowNormal = isWater ? tbn * renderWaveNormal(worldPosition + cameraPosition) : shadowNormal;

  color1.x = EncodeNormal(shadowNormal) * 0.5 + 0.5;
  color1.y = lmcoord.y * 0.5 + 0.5;
  color1.z = float(isWater) * 0.5 + 0.5;
  color1.w = 1.0;
}
#endif
