#include "/program/light/common/atmosphere/constant.glsl"
#include "/program/light/common/atmosphere/phase.glsl"
#include "/program/light/common/atmosphere/transmittance.glsl"
#include "/program/light/common/atmosphere/spot.glsl"

vec2 rsi(vec3 position, vec3 direction, float radius) {
    float PoD = dot(position, direction);
    float radiusSquared = radius * radius;

    float delta = PoD * PoD + radiusSquared - dot(position, position);
    if (delta < 0.0) return vec2(-1.0);
          delta = sqrt(delta);

    return -PoD + vec2(-delta, delta);
}

#include "/program/light/common/atmosphere/volumetricClouds.glsl"

vec3 renderAtmosphericScattering(vec3 color, vec3 vector, vec3 sunVector, vec3 moonVector, vec3 upVector, out vec2 pid, out vec3 transmittance, const int steps){
  const int jSteps = 4;

  vec3 viewPosition = (earthSize + eyeAltitude) * upVector;

  vec2 aid = rsi(viewPosition, vector, atmosphereRadius);
  if (aid.y < 0.0) {transmittance = vec3(1.0); return vec3(0.0);}

  pid = rsi(viewPosition, vector, earthSize * 0.998);
	bool planetIntersected = pid.y >= 0.0;

  vec2 sd = vec2((planetIntersected && pid.x < 0.0) ? pid.y : max(aid.x, 0.0), (planetIntersected && pid.x > 0.0) ? pid.x : aid.y);

	float stepSize = (sd.y - sd.x) / steps;
	vec3 increment = vector * stepSize;
	vec3 position  = vector * sd.x + (increment * 0.2 + viewPosition);

  vec2 phaseSun = phase_Sky(dot(vector, sunVector), sky_mieg);
	vec2 phaseMoon = phase_Sky(dot(vector, moonVector), sky_mieg);

	vec3 scatteringSun = vec3(0.0);
	vec3 scatteringMoon = vec3(0.0);
	vec3 scatteringAmbient = vec3(0.0);
	transmittance = vec3(1.0);

  for(int i = 0; i < steps; ++i, position += increment) {
    vec3 density = renderDensity(length(position));
    if (density.y > 1e35) break;

		vec3 stepAirmass = density * stepSize;
		vec3 stepOpticalDepth = attenuationCoeff * stepAirmass;

    vec3 stepTransmittance = clamp01(exp2(-stepOpticalDepth * rLOG2));
    vec3 stepTransmittedFraction = clamp01((stepTransmittance - 1.0) / -stepOpticalDepth);
    vec3 stepScatteringVisible = transmittance * stepTransmittedFraction;

    scatteringSun += scatteringCoeff * (stepAirmass.xy * phaseSun) * stepScatteringVisible * renderTransmittance(position, sunVector,  3);
		scatteringMoon += scatteringCoeff * (stepAirmass.xy * phaseMoon) * stepScatteringVisible * renderTransmittance(position, moonVector, 3);

    scatteringAmbient += scatteringCoeff * stepAirmass.xy * stepScatteringVisible;

		transmittance *= stepTransmittance;
  }

  vec3 scattering = scatteringSun * sunLux + scatteringMoon * moonLux + scatteringAmbient * skylight;
       scattering = mix(scattering, vec3(renderLuminance701(scattering)), wetness * 0.95);

  return max0(mix(color * transmittance, vec3(0.0), vec3(!planetIntersected)) + scattering);
}
