vec3 renderDensity(float centerDistance) {
	vec2 rayleighMie = exp(centerDistance * -rScaleHeigh + rScaledEarthSize);

  float ozone = exp2(-max(0.0, (35000.0 - centerDistance) - earthSize) / 5000.0)
	            * exp2(-max(0.0, (centerDistance - 35000.0) - earthSize) / 15000.0);

	return vec3(rayleighMie, ozone);
}

vec3 renderAirmass(vec3 position, vec3 direction, float rayLength, const float steps) {
	float stepSize  = rayLength / steps;
	vec3 increment = direction * stepSize;
	     position += increment * 0.5;

	vec3 airmass = vec3(0.0);
	for (int i = 0; i < steps; i++, position += increment) {
		airmass += renderDensity(length(position));
	}

	return airmass * stepSize;
}

vec3 renderAirmass(vec3 position, vec3 direction, const float steps) {
	float rayLength = dot(position, direction);
	      rayLength = rayLength * rayLength + atmosphereRadiusSquared - dot(position, position);
		  if (rayLength < 0.0) return vec3(0.0);
	      rayLength = sqrt(rayLength) - dot(position, direction);

	return renderAirmass(position, direction, rayLength, steps);
}

vec3 renderOpticalDepth(vec3 position, vec3 direction, float rayLength, const float steps) {
	return attenuationCoeff * renderAirmass(position, direction, rayLength, steps);
}

vec3 renderOpticalDepth(vec3 position, vec3 direction, const float steps) {
	return attenuationCoeff * renderAirmass(position, direction, steps);
}

vec3 renderTransmittance(vec3 position, vec3 direction, const float steps) {
	return max0(exp2(-renderOpticalDepth(position, direction, steps) * rLOG2));
}
