struct DataBuffer{
  vec3 color;
  vec3 normal;
  vec3 worldNormal;

  vec2 depth;
  vec2 lightmap;

  float ID;
  float roughness;
  float f0;
  float dither;

  mat2x3 position;
  mat2x3 worldPosition;
  mat2x3 vector;
  mat2x3 worldVector;
};

vec3 renderNormal(vec2 coord){
  return DecodeNormal(gbufferModelView, texture2D(colortex2, coord).x);
}

vec2 renderDepth(vec2 coord){
  return vec2(texture2D(depthtex0, coord).x, texture2D(depthtex1, coord).x);
}

DataBuffer renderBuffer(){
  DataBuffer data;

  vec4 color0 = texture2D(colortex0, coord);
  vec4 color2 = texture2D(colortex2, coord);

  data.color = toGamma(color0.rgb);
  data.normal = renderNormal(coord);
  data.worldNormal = mat3(gbufferModelViewInverse) * data.normal;

  data.depth = renderDepth(coord);

  data.ID = (1.0 - DecodeVec2(color2.w).x) * 32.0;

  data.position = mat2x3(renderViewPosition(coord, data.depth.x),
                         renderViewPosition(coord, data.depth.y));

  data.worldPosition = mat2x3(renderWorldPosition(data.position[0]),
                              renderWorldPosition(data.position[1]));

  data.vector = mat2x3(normalize(data.position[0]),
                       normalize(data.position[1]));

  data.worldVector = mat2x3(mat3(gbufferModelViewInverse) * data.vector[0],
                            mat3(gbufferModelViewInverse) * data.vector[1]);

  data.lightmap = vec2(DecodeVec2(color2.y).x, DecodeVec2(color2.y).y);

  data.roughness = DecodeVec2(color2.z).x;
  data.f0 = DecodeVec2(color2.z).y;

  #ifdef TEMPORAL_AA
  data.dither = fract(frameTime / 8.0 + bayer64(gl_FragCoord.xy));
  #else
	data.dither = bayer64(gl_FragCoord.xy);
  #endif

  return data;
}
