#include "/program/light/common/shadow.glsl"
#include "/program/light/common/ao.glsl"
#include "/program/light/common/gi.glsl"

vec3 renderSkylight(float lightmap){
  vec3 skylighting = FromSH(skySH[0], skySH[1], skySH[2], worldUpVector) * PI;
       skylighting = mix(skylighting, vec3(renderLuminance701((sunlight + moonlight) * transitionFading)), vec3(rPI) * 0.15);   //Fake bounced
       skylighting *= lightmap * exp2(6.7 * (lightmap - 1.0));

  return skylighting * SHADOW_BRIGHTNESS;
}

float renderTorchlight(float lightmap){
  lightmap = pow(lightmap, 1.0 / 0.25);
  lightmap = pow2(lightmap) * 5.0;
  lightmap += pow(lightmap, 0.4) * 0.1;

  return lightmap;
}

#if defined deferred1
#if defined world0
vec3 renderDiffuse(vec3 color, vec3 normal, vec3 worldNormal, vec3 position, vec3 worldPosition, vec3 vector, vec3 worldVector, vec2 lightmap, vec2 depth, float dither, float roughness, float f0, Material material){

  #ifdef CLOUD_SHADOW
  float cloudShadow = renderCloudShadow(worldPosition + cameraPosition, worldLightVector);
  #else
  float cloudShadow = 1.0;
  #endif

  worldNormal = material.isVegetation ? vec3(0.0, 1.0, 0.0) : worldNormal;

  float diffuse = renderLambertDiffuse(worldNormal, worldLightVector);	//Simple Lambert diffuse
  #if !defined VOLUME_CLOUDS || !defined CLOUD_SHADOW
        diffuse *= mix(1.0, 0.2, wetness);
  #endif

  vec3 shadow = renderShadow(worldPosition, normal, lightmap.y, dither, material.isVegetation) * cloudShadow;

  float ao = renderAmbientOcclusion(position, normal, dither);

  vec3 final = vec3(0.0);
       final += shadow * diffuse * (sunlight + moonlight) * (0.5 * transitionFading);
       final += renderSkylight(lightmap.y) * ao;
       final += renderTorchlight(lightmap.x) * torchlight;
       //final += renderGloablIllumination(shadow, worldPosition, worldNormal, dither, lightmap.y, material.isVegetation) * (sunlight + moonlight) * cloudShadow;

  vec3 ggx = renderSpecularGGX(worldNormal, worldLightVector, worldVector, f0, roughness) * rPI * (sunlight + moonlight) * shadow * (0.5 * transitionFading);

  return renderBlendMaterial(final * color, ggx, color, f0);
}
#else
vec3 renderDiffuse(vec3 color, vec3 normal, vec3 worldNormal, vec3 position, vec2 lightmap, float dither){
  float ao = renderAmbientOcclusion(position, normal, dither);

  vec3 final = vec3(0.0);
       final += renderSkylight(lightmap.y) * ao;
       final += renderTorchlight(lightmap.x) * torchlight;

  return final * color;
}
#endif
#endif
