
vec3 renderGloablIllumination(vec3 shadow, vec3 worldPosition, vec3 worldNormal, float dither, float lightmap, bool isVeg){
  const int steps = 8;
  const float rSteps = 1.0 / steps;

  const float radius = 50;
  const float inverseRadius = 1.0 / radius;
  const float radiusSquared = radius * radius;

  vec3 shadowPosition = transMAD(shadowMatrix, worldPosition);
  vec3 shadowSpaceNormal = mat3(shadowModelView) * worldNormal * vec3(1.0, 1.0, -1.0);

  vec3 indirect = vec3(0.0);

  #ifdef TEMPORAL_AA
    dither += fract(frameTimeCounter / 16.0 + bayer64(gl_FragCoord.xy));
  #endif

  for(int i = 0; i < steps; ++i){
    vec3 sampleOffset = circlemapL((dither + i) * rSteps, 4096.0 * steps) * (radius / 2048.0);

    vec3 sampleCoord = shadowPosition + sampleOffset;
    vec3 sampleBias = renderShadowDistortPosition(sampleCoord) * 0.5 + 0.5;

    vec2 depth = vec2(texture2D(shadowtex0, sampleBias.xy).x, texture2D(shadowtex1, sampleBias.xy).x) - 0.00005;

    vec4 color0 = texture2D(shadowcolor0, sampleBias.xy);
    vec4 color1 = texture2D(shadowcolor1, sampleBias.xy);

    vec3 samplePosition = vec3(sampleCoord.xy, depth.y * 8.0 - 4.0) * sampleCoord.z - shadowPosition;
    float sampleSquared = dot(samplePosition, samplePosition);

    if(sampleSquared > radiusSquared) continue;

    vec3 sampleVector = samplePosition * inversesqrt(sampleSquared);

    float NdotL = isVeg ? 1.0 : clamp01(dot(sampleVector, shadowSpaceNormal));
    if(NdotL <= 0.0) continue;

    vec3 sampleNormal = color1.xyz * 2.0 - 1.0;
         sampleNormal.xy -= sampleNormal.xy;

    float SdotL = clamp01(dot(sampleVector, sampleNormal));
    if(SdotL <= 0.0) continue;

    float falloff = 1.0 / (sampleSquared * 1000.0 + 16.0);

    float skylight = pow6((color0.a * 2.0 - 1.0) - lightmap);
          skylight = clamp01(0.05 / max(skylight, 0.005));

    indirect += toGamma(color0.rgb) * SdotL * NdotL * falloff * (skylight * 0.8 + 0.2);
  }

  float value = length(indirect);

  indirect = pow(value, 1.25) * normalize(indirect + 0.0001) * 0.8;

  return indirect * sqrt(radiusSquared * rSteps);
}
