float renderHardShadows(float shadowDepth, vec3 shadowPosition, float bias) {
  if(shadowPosition.z >= 1.0) return 1.0;
  return 1.0 - fstep(shadowDepth, shadowPosition.z - bias);
}

vec3 renderShadow(vec3 worldPosition, vec3 normal, float lightmap, float dither, bool isVeg){
  const int steps = 4;
  const float rSteps = 1.0 / steps;

  vec3 shadowPosition = transMAD(shadowMatrix, worldPosition);
  vec3 shadowSpaceNormal = mat3(shadowModelView) * mat3(gbufferModelViewInverse) * normal;

  if(any(greaterThanEqual(renderShadowDistortPosition(shadowPosition) * 0.5 + 0.5, vec3(1.0))) || any(lessThanEqual(renderShadowDistortPosition(shadowPosition) * 0.5 + 0.5, vec3(0.0)))) return vec3(1.0);

  float NdotL = isVeg ? rPI : dot(normal, lightVector);
  if(NdotL <= 0.0) return vec3(1.0);

  float bias = sqrt(sqrt(1.0 - NdotL * NdotL) / NdotL) * rShadowMapResolution;
        bias = bias * renderShadowDistort(shadowPosition.xy) * 0.20;

  vec3 shadow = vec3(0.0);
  float penumbra = 0.0;

  float maxSpread = 7.0;
  float angle = radians(0.533333) * 4.0;

  for(int i = 0; i < steps; ++i){
    vec3 sampleOffset = circlemapL((dither + i) * rSteps, 4096.0 * steps);
         sampleOffset.z *= maxSpread * rShadowMapResolution;

    vec3 samplePosition = vec3(sampleOffset.xy, -bias) * sampleOffset.z + shadowPosition;
         samplePosition = renderShadowDistortPosition(samplePosition) * 0.5 + 0.5;

    float depth = texture2D(shadowtex1, samplePosition.xy).x;

    penumbra = max(penumbra, (samplePosition.z - bias) - depth);
  }

  penumbra = min(penumbra * angle, maxSpread);

  for(int i = 0; i < steps; ++i){
    vec3 sampleOffset = circlemapL((dither + i) * rSteps, 4096.0 * steps);
    #if SHADOW_TYPE == 1
         sampleOffset.z *= penumbra;
    #else
         sampleOffset.z *= rShadowMapResolution * 0.2;
    #endif

    vec3 samplePosition = vec3(sampleOffset.xy, -bias) * sampleOffset.z + shadowPosition;
         samplePosition = renderShadowDistortPosition(samplePosition) * 0.5 + 0.5;

    vec2 depth = vec2(texture2D(shadowtex0, samplePosition.xy).x, texture2D(shadowtex1, samplePosition.xy).x);

    vec4 color0 = texture2D(shadowcolor0, samplePosition.xy);
    vec4 color1 = texture2D(shadowcolor1, samplePosition.xy);

    //bias = (depth.x == depth.y && !isVeg) ? dot(DecodeNormal(color1.x * 2.0 - 1.0), shadowSpaceNormal) > 0.01 ? bias : 0.0 : bias;

    float shadow0 = renderHardShadows(depth.x, samplePosition, bias);
    float shadow1 = renderHardShadows(depth.y, samplePosition, bias);

    float surface0 = (depth.x * 2.0 - 1.0) * shadowProjectionInverse[2].z + shadowProjectionInverse[3].z;
	  float surface1 = (depth.y * 2.0 - 1.0) * shadowProjectionInverse[2].z + shadowProjectionInverse[3].z;

		float waterDepth = mix(0.0, (surface0 - surface1), color0.a);

    vec3 waterTransmittance = exp2(-waterAbsorptionCoefficient * waterDepth * rLOG2 * 1.11);

    #ifdef COLOR_SHADOW
    shadow += mix(vec3(shadow0), toGamma(color0.xyz), clamp01(shadow1 - shadow0)) * waterTransmittance;
    #else
    shadow += shadow1 * waterTransmittance;
    #endif
  }
  //shadow *= clamp01(lightmap * 100.0); //Light leak fix in underground

  return shadow * rSteps;
}
