vec3 renderRaytrace(vec3 viewPosition, vec3 reflectVector, vec3 p, vec3 sky, float lightmap){

	int raySteps = SPEC_QUALITY;
	int refinements = 8;

  vec3 direction = normalize(renderScreenPosition(viewPosition + reflectVector) - p);

	float maxLength = 1.0 / raySteps;
	float minLength = 0.1 / raySteps;

	float stepLength = minLength;
	float stepWeight = 1.0 / abs(direction.z);

	float depth = p.z;
	bool rayHit = false;

  while(depth >= p.z){
		stepLength = clamp((depth - p.z) * stepWeight, minLength, maxLength);
		p = direction * stepLength + p;

		if (clamp01(p) != p) return sky * lightmap;

    depth = renderDepth(p.xy).x;
	}

  while (--refinements > 0) {

		vec3 reflectp = p + direction * clamp((depth - p.z) * stepWeight, -stepLength, stepLength);
		float reflectDepth = renderDepth(reflectp.xy).x;
    bool ray = reflectDepth < reflectp.z;

    p = ray ? reflectp : p;
    depth = ray ? reflectDepth : depth;

		stepLength *= 0.5;
	}

  bool visible = abs(p.z - depth) * min(stepWeight, 400.0) <= maxLength;

	return visible ? max0(DecodeRGBE8(texture2D(colortex1, p.xy))) : sky * lightmap;
}

vec3 renderScreenSpaceReflection(vec3 position, vec3 vector, vec3 normal, vec3 sky, float lightmap){
  const int rayTraceQuality = 80;
	const float rQuality = 1.0 / rayTraceQuality;
  const float stepLength = 300.0 * rQuality;

  vec3 increment = normalize(reflect(vector, normal)) * stepLength;

  float depth = renderDepth(renderScreenPosition(position).xy).x;

  vec3 p = vec3(coord, depth);

  vec3 color = vec3(0.0);

  for(int i = 0; i < rayTraceQuality; ++i){
    if(clamp01(p) != p) continue;

    position += increment;

    p = renderScreenPosition(position);
    depth = renderDepth(p.xy).x;

    if(p.z > depth) break;
  }

  bool outBounds = length(position) < rayTraceQuality * stepLength;
  bool visible = abs(p.z - depth) * length(position) < rayTraceQuality && outBounds;

  color = visible ? DecodeRGBE8(texture2D(colortex1, p.xy)) : sky * lightmap;

  return color;
	//return visible ? max0(DecodeRGBE8(texture2D(colortex1, p.xy))) : sky * lightmap;
}
