//Simple raymarch from robobo1221
//https://www.shadertoy.com/view/XtdBDX

#include "/program/light/common/specular/raytracer.glsl"

vec3 renderRoughSpecular(const float i, const float alpha2, const int steps) {
  float x = (alpha2 * i) / (1.0 - i);
  float y = i * float(steps) * 64.0 * 64.0 * goldenAngle;

  float c = inversesqrt(x + 1.0);
  float s = sqrt(x) * c;

  return vec3(cos(y) * s, sin(y) * s, c);
}

#if defined composite1
vec3 renderRoughnessSpecular(vec3 color, DataBuffer data){
  if(data.f0 < 0.005) return color;

  const int steps = SPEC_ROUGH_QUALITY;
  const float rSteps = 1.0 / steps;

  vec3 reflection = vec3(0.0);
  vec3 fresnel = vec3(0.0);

  #ifdef ENABLE_TAA
  dither += fract(frameTimeCounter / 8.0 + bayer64(gl_FragCoord.xy));
  #endif

  float specularOffset = data.dither * rSteps;

  vec3 shadowPosition = renderShadowDistortPosition(transMAD(shadowMatrix, data.worldPosition[1])) * 0.5 + 0.5;
  float shadow = float(texture2DLod(shadowtex1, shadowPosition.xy, 0).x > shadowPosition.z - rShadowMapResolution * 0.15);

  data.lightmap.y = pow2(data.lightmap.y);

  vec3 c0 = cross(data.normal, vec3(0.0, 0.0, 1.0));
  vec3 c1 = cross(data.normal, vec3(0.0, 1.0, 0.0));

  vec3 tangent = length(c0) > length(c1) ? c0 : c1;
  mat3 tbn = mat3(tangent, cross(data.normal, tangent), data.normal);

  if(data.depth.x < 1.0){
    float alpha2 = pow4(data.roughness);
    float VdotN = clamp01(dot(data.normal, -data.vector[0]));

    if(data.roughness >= 0.03){
      for(int i = 0; i < steps; ++i){
        vec3 sampleOffset = tbn * renderRoughSpecular((float(i) + specularOffset) * rSteps, alpha2, steps);

        float VdotS = abs(dot(sampleOffset, -data.vector[0]));

        vec3 vector = ((2.0 * VdotS) * sampleOffset + data.vector[0]);
        vec3 worldVector = mat3(gbufferModelViewInverse) * vector;

        float NdotV = clamp01(dot(data.normal, vector));
        float g = ExactCorrelatedG2(alpha2, VdotN, NdotV);

        vec3 sky = DecodeRGBE8(texture2D(colortex3, renderUnprojectSky(-worldVector) * 0.5));
             //sky += DecodeRGBE8(texture2D(colortex6, coord));

        fresnel += Fresnel(data.f0, 1.0, VdotS);
        //reflection += renderScreenSpaceReflection(data.position[0], vector, data.normal, sky, data.lightmap.y) * g;
        reflection += renderRaytrace(data.position[0], vector, vec3(coord, data.depth.x), sky, data.lightmap.y);
      }
      fresnel *= rSteps;
      reflection *= rSteps;
    }
    reflection *= fresnel;
    reflection += renderSpecularBRDF(data.normal, lightVector, data.vector[0], data.f0, alpha2) * shadow * (sunlight + moonlight);

    #ifdef SPECULAR
    return renderBlendMaterial(color * (1.0 - fresnel), reflection, data.color, data.f0);
    #else
    return color;
    #endif
  } else {
    return color;
  }
}
#elif defined composite0
vec3 renderTranslucentSpecular(vec3 color, DataBuffer data){
  if(data.f0 < 0.005) return color;

  vec3 reflection = vec3(0.0);
  vec3 fresnel = vec3(0.0);

  vec3 shadowPosition = renderShadowDistortPosition(transMAD(shadowMatrix, data.worldPosition[1])) * 0.5 + 0.5;
  float shadow = float(texture2DLod(shadowtex1, shadowPosition.xy, 0).x > shadowPosition.z - rShadowMapResolution * 0.15);

  data.lightmap.y = pow2(data.lightmap.y);

  if(data.depth.x < 1.0){
    float alpha2 = pow4(data.roughness);
    float VdotN = clamp01(dot(data.normal, -data.vector[0]));

    if(data.roughness <= 0.03){
      vec3 vector = ((2.0 * VdotN) * data.normal + data.vector[0]);
      vec3 worldVector = mat3(gbufferModelViewInverse) * vector;

      vec3 sky = isEyeInWater == 0 ? DecodeRGBE8(texture2D(colortex3, renderUnprojectSky(-worldVector) * 0.5)) : vec3(0.0);
           //sky += DecodeRGBE8(texture2D(colortex6, coord));

      fresnel = Fresnel(data.f0, 1.0, VdotN);
      //reflection = renderScreenSpaceReflection(data.position[0], vector, data.normal, sky, data.lightmap.y);
      reflection = renderRaytrace(data.position[0], vector, vec3(coord, data.depth.x), sky, data.lightmap.y);
    }

    reflection *= fresnel;
    reflection += renderSpecularGGX(data.normal, lightVector, data.vector[0], data.f0, alpha2) * shadow * (sunlight + moonlight);

    return renderBlendMaterial(color * (1.0 - fresnel), reflection, data.color, data.f0);
  } else {
    return color;
  }
}

#endif
