out vec2 coord;

out vec3 sunVector, moonVector, lightVector, upVector;
out vec3 worldSunVector, worldMoonVector, worldLightVector, worldUpVector;
out vec3 sunlight, moonlight, skylight , torchlight;

out float sunRise, sunNoon, sunSet, dayTime;
out float moonRise, moonNoon, moonSet, moonTime;

out float ambientFog;
out float transitionFading;

out mat4 shadowMatrix;
out mat3x4 skySH;

#include "/program/light/common/atmosphere/atmosphericScattering.glsl"

void renderSkylight(vec3 sunVector, vec3 moonVector, vec3 upVector, vec2 planetSphere, vec3 transmittance){
  const int latSamples = 5;
	const int lonSamples = 5;

	const float rLatSamples = 1.0 / latSamples;
	const float rLonSamples = 1.0 / lonSamples;
	const float sampleCount = rLatSamples * rLonSamples;

	float latitudeSize = rLatSamples * TAU;
	float longitudeSize = rLonSamples * PI;

  const float offset = 0.1;

  vec4 shR = vec4(0.0), shG = vec4(0.0), shB = vec4(0.0);

  for (int i = 0; i < latSamples; ++i) {
		float latitude = float(i) * latitudeSize;

		for (int j = 0; j < lonSamples; ++j) {
			float longitude = float(j) * longitudeSize;

			float c = cos(latitude);
			vec3 kernel = vec3(c * cos(longitude), sin(latitude), c * sin(longitude));

      vec3 sky = renderAtmosphericScattering(vec3(0.0), normalize(kernel + vec3(0.0, offset, 0.0)), sunVector, moonVector, upVector, planetSphere, transmittance, 8) * 1.2;

      shR += ToSH(sky.r, kernel);
			shG += ToSH(sky.g, kernel);
			shB += ToSH(sky.b, kernel);
    }
  }
  skySH = mat3x4(shR, shG, shB) * sampleCount;
}

void main(){
  coord = gl_Vertex.xy;
  gl_Position = vec4(gl_Vertex.xy * 2.0 - 1.0, 0.0, 1.0);

  sunVector = sunPosition * 0.01;
  moonVector = -sunVector;
  lightVector = shadowLightPosition * 0.01;
  upVector = upPosition * 0.01;

  worldSunVector = mat3(gbufferModelViewInverse) * sunVector;
  worldMoonVector = mat3(gbufferModelViewInverse) * moonVector;
  worldLightVector = mat3(gbufferModelViewInverse) * lightVector;
  worldUpVector = mat3(gbufferModelViewInverse) * upVector;

  shadowMatrix = shadowProjection * shadowModelView;

  vec3 absorb = vec3(0.0);
  vec2 sphere = vec2(0.0);

  torchlight = blackbody(TORCHLIGHT_TEMP) * 10.0 * TORCHLIGHT_BRIGHTNESS;

  sunlight = renderTransmittance(vec3(0.0, earthSize, 0.0), worldSunVector, 8) * sunLux;
  moonlight = renderDesaturation(renderTransmittance(vec3(0.0, earthSize, 0.0), worldMoonVector, 8) * moonLux);

  skylight = vec3(0.0);
  skylight = renderAtmosphericScattering(vec3(0.0), worldUpVector, worldSunVector, worldMoonVector, worldUpVector, sphere, absorb, 8);

  renderSkylight(worldSunVector, worldMoonVector, worldUpVector, sphere, absorb);

  sunRise = pow4(clamp01(dot(worldSunVector, vec3(1.0, 0.0, 0.0))));
  sunNoon = pow4(clamp01(dot(worldSunVector, vec3(0.0, 1.0, 0.0))));
  sunSet = pow4(clamp01(dot(worldSunVector, vec3(0.0, 0.0, 1.0))));

  moonRise = pow4(clamp01(dot(worldMoonVector, vec3(1.0, 0.0, 0.0))));
  moonNoon = pow4(clamp01(dot(worldMoonVector, vec3(0.0, 1.0, 0.0))));
  moonSet = pow4(clamp01(dot(worldMoonVector, vec3(0.0, 0.0, 1.0))));

  dayTime = sunRise + sunNoon + sunSet;
  moonTime = moonRise + moonNoon + moonSet;

  ambientFog = eyeBrightnessSmooth.y / 240.0;
  transitionFading = clamp01(clamp01(float(worldTime - 23215) / 50.0) + (1.0 - clamp01(float(worldTime - 12735) / 50.0)) + clamp01(float(worldTime - 12925) / 50.0) * (1.0 - clamp01(float(worldTime - 23075) / 50.0)));
}
