#if defined vertex
#include "/program/light/common/vector.glsl"
#elif defined fragment
/* DRAWBUFFERS:5 */
layout(location = 0) out vec4 color5;

in vec2 coord;

in vec3 sunVector, moonVector, lightVector, upVector;
in vec3 worldSunVector, worldMoonVector, worldLightVector, worldUpVector;
in vec3 sunlight, moonlight, skylight, torchlight;

in float sunRise, sunNoon, sunSet, dayTime;
in float moonRise, moonNoon, moonSet, moonTime;

in float ambientFog;
in float transitionFading;

in mat4 shadowMatrix;

in mat3x4 skySH;

#include "/program/light/common/buffer.glsl"
#include "/program/utility/material.glsl"

#include "/program/light/common/atmosphere/atmosphericScattering.glsl"
#include "/program/light/common/diffuse.glsl"

#include "/program/light/common/specular/reflection.glsl"

#include "/program/light/common/atmosphere/volumetricRay.glsl"

vec3 renderTranslucent(vec3 color, DataBuffer data, Material material){
  if(material.isWater) return color;

  vec4 color0 = texture2D(colortex0, coord);

  vec3 diffuse = DecodeRGBE8(texture2D(colortex1, coord));
       diffuse = pow(length(diffuse), 1.0 / gamma) * normalize(diffuse + 0.00001);  //Translucent absorbsion

  if(material.isTranslucent){
    return mix(color * mix(vec3(0.0), toGamma(color0.rgb), fsign(color0.a)), diffuse * rPI, color0.a);
  } else {
    return color;
  }
}

vec3 renderNetherFog(vec3 color, vec3 worldPosition){
  float fogFactor = exp2(-length(worldPosition) * 0.001);
  vec3 fog = mix(vec3(1.0), vec3(0.9, 0.05, 0.0) * pow2(TAU), sqrt(fogFactor));

  return mix(fog, color, fogFactor);
}

void main(){
  DataBuffer data = renderBuffer();
  Material material = renderMaterial(data.ID, data.depth);

  vec3 color = DecodeRGBE8(texture2D(colortex1, coord));
       color = material.isTranslucent && (!material.isWater || isEyeInWater == 0)? renderVolumetricRay(color, data.worldPosition[0], data.worldPosition[1], data, material) : color;
       color = renderTranslucent(color, data, material);
       color = renderRoughnessSpecular(color, data);
       color = isEyeInWater == 0 ? renderVolumetricRay(color, gbufferModelViewInverse[3].xyz, data.worldPosition[0], data, material) : color;
  #if !defined world0
       color = renderNetherFog(color, data.worldPosition[1]);
  #endif

  color5.rgb = EncodeColor(color);
  color5.a = texture2D(colortex5, vec2(0.75)).a;
}
#endif
