#if defined vertex
#include "/program/light/common/vector.glsl"
#elif defined fragment
/* DRAWBUFFERS:36 */
layout(location = 0) out vec4 color3;
layout(location = 1) out vec4 color6;

in vec2 coord;

in vec3 sunVector, moonVector, lightVector, upVector;
in vec3 worldSunVector, worldMoonVector, worldLightVector, worldUpVector;
in vec3 sunlight, moonlight, skylight, torchlight;

in float sunRise, sunNoon, sunSet, dayTime;
in float moonRise, moonNoon, moonSet, moonTime;

in float ambientFog;
in float transitionFading;

in mat4 shadowMatrix;

in mat3x4 skySH;

#include "/program/light/common/buffer.glsl"
#include "/program/utility/material.glsl"

#include "/program/light/common/atmosphere/atmosphericScattering.glsl"
#include "/program/light/common/diffuse.glsl"

#include "/program/light/common/atmosphere/volumetricRay.glsl"

vec3 renderAtmosphere(DataBuffer data, Material material){
  vec3 atmosphere = vec3(0.0);
  vec3 transmittance = vec3(1.0);
  vec2 sphere = vec2(0.0);

	if(any(greaterThan(coord, vec2(0.501)))) return vec3(0.0);

	vec3 position = mat3(gbufferModelView) * (renderProjectSky((coord * 2.0) - (jitter * 0.5)));
	vec3 worldPosition = mat3(gbufferModelViewInverse) * position;

	vec3 vector = normalize(-position);
  vec3 worldVector = normalize(-worldPosition);

  atmosphere = renderAtmosphericScattering(atmosphere, vector, sunVector, moonVector, upVector, sphere, transmittance, 15);

  return atmosphere;
}

vec3 renderVolumetric(DataBuffer data, Material material){
  vec3 volumetric = vec3(0.0);

  //if(any(greaterThan(coord, vec2(0.501)))) return vec3(0.0);

	vec3 position = mat3(gbufferModelView) * (renderProjectSky(coord * 2.0));
	vec3 worldPosition = mat3(gbufferModelViewInverse) * position;

	vec3 vector = normalize(-position);
  vec3 worldVector = normalize(-worldPosition);

  float VdotS = dot(data.vector[1], sunVector);

  volumetric = renderVolumeClouds(volumetric, data.worldPosition[1], data.worldVector[1], VdotS, 1.0, data.dither, VOLUME_CLOUDS_QUALITY);

  return volumetric;
}

void main(){
  #if defined world0
    DataBuffer data = renderBuffer();
    Material material = renderMaterial(data.ID, data.depth);

    color3 = EncodeRGBE8(renderAtmosphere(data, material));
    color6 = EncodeRGBE8(renderVolumetric(data, material));
  #endif
}
#endif
