#if defined vertex
#include "/program/light/common/vector.glsl"
#elif defined fragment
/* DRAWBUFFERS:1 */
layout(location = 0) out vec4 color1;

in vec2 coord;

in vec3 sunVector, moonVector, lightVector, upVector;
in vec3 worldSunVector, worldMoonVector, worldLightVector, worldUpVector;
in vec3 sunlight, moonlight, skylight, torchlight;

in float sunRise, sunNoon, sunSet, dayTime;
in float moonRise, moonNoon, moonSet, moonTime;

in float ambientFog;
in float transitionFading;

in mat4 shadowMatrix;

in mat3x4 skySH;

#include "/program/light/common/buffer.glsl"
#include "/program/utility/material.glsl"

#include "/program/light/common/atmosphere/atmosphericScattering.glsl"
#include "/program/light/common/diffuse.glsl"

vec3 renderAtmosphere(vec3 color, vec3 normal, vec3 vector, float lightmap, Material material){

  vec2 uv = renderUnprojectSky(-vector);

  vec3 atmosphere = DecodeRGBE8(texture2D(colortex3, uv * 0.5));

  vec3 sun = renderSun(vector, worldSunVector) * sunlight;
  vec3 moon = renderMoon(vector, worldMoonVector) * moonlight;

  atmosphere += (sun + moon) * (1.0 - wetness);
  atmosphere += renderStars(vector, worldMoonVector) * (1.0 - wetness);
  atmosphere += DecodeRGBE8(texture2DLod(colortex6, coord, 2));

  return material.isSky ? atmosphere : color;
}

void main(){
  DataBuffer data = renderBuffer();
  Material material = renderMaterial(data.ID, data.depth);

  float VdotS = dot(data.vector[1], lightVector);

  #if defined world0
    vec3 color = renderDiffuse(data.color, data.normal, data.worldNormal, data.position[1], data.worldPosition[1], data.vector[1], data.worldVector[1], data.lightmap, data.depth, data.dither, pow2(data.roughness), data.f0 * 0.98 + 0.02, material);
         color = renderAtmosphere(color, data.normal, data.worldVector[1], data.lightmap.y, material);
         //color = renderVolumeClouds(color, data.worldPosition[1], data.worldVector[1], VdotS, data.depth.y, data.dither, VOLUME_CLOUDS_QUALITY);
  #else
    vec3 color = renderDiffuse(data.color, data.normal, data.worldNormal, data.position[1], data.lightmap, data.dither);
         color = material.isSky ? vec3(0.9, 0.2, 0.0) : color;
  #endif

  color1 = EncodeRGBE8(color);
}
#endif
