#if defined vertex
#include "/program/light/common/vector.glsl"
#elif defined fragment
/* DRAWBUFFERS:1 */
layout(location = 0) out vec4 color1;

const bool colortex1MipmapEnabled = true;

in vec2 coord;

in vec3 sunVector, moonVector, lightVector, upVector;
in vec3 worldSunVector, worldMoonVector, worldLightVector, worldUpVector;
in vec3 sunlight, moonlight, skylight, torchlight;

in float sunRise, sunNoon, sunSet, dayTime;
in float moonRise, moonNoon, moonSet, moonTime;

in float ambientFog;
in float transitionFading;

in mat4 shadowMatrix;

in mat3x4 skySH;

#include "/program/light/common/buffer.glsl"
#include "/program/utility/material.glsl"

#include "/program/light/common/atmosphere/atmosphericScattering.glsl"

#include "/program/light/common/specular/reflection.glsl"

#include "/program/light/common/atmosphere/volumetricRay.glsl"

void renderRefraction(inout vec2 coord, inout DataBuffer data, bool isTranslucent){
  if(isTranslucent){
    vec3 flatNormal = clamp(normalize(cross(dFdx(data.position[0]), dFdy(data.position[0]))), -1.0, 1.0);
    vec3 direction = refract(data.vector[0], (data.normal - flatNormal), 0.75);

    vec3 refractPosition = direction * abs(distance(data.worldPosition[0], data.worldPosition[1])) + data.position[0];
         refractPosition = renderScreenPosition(refractPosition);
         refractPosition.z = texture2D(depthtex1, refractPosition.xy).x;

    if(refractPosition.z > texture2D(depthtex0, refractPosition.xy).x){
      coord = refractPosition.xy;
      data.depth.y = refractPosition.z;

      data.position[1] = renderViewPosition(refractPosition);
      data.worldPosition[1] = renderWorldPosition(data.position[1]);

      data.vector[0] = inversesqrt(dot(data.position[1], data.position[1])) * data.position[1];
      data.worldVector[0] * mat3(gbufferModelViewInverse) * data.vector[0];
    }
  }
}

void main(){
  DataBuffer data = renderBuffer();
  Material material = renderMaterial(data.ID, data.depth);

  vec2 refractCoord = coord;

  #ifdef WATER_REFRACTION
  renderRefraction(refractCoord, data, material.isTranslucent);
  #endif

  vec3 waterOpaque = isEyeInWater == 1 ? data.worldPosition[0] : data.worldPosition[1];
  vec3 waterDeep = isEyeInWater == 1 ? gbufferModelViewInverse[3].xyz : data.worldPosition[0];

  vec3 color = DecodeRGBE8(texture2DLod(colortex1, refractCoord, 0));
       color = material.isWater || isEyeInWater == 1 ? renderVolumetricRayWater(color, waterDeep, waterOpaque, data, material) : color;
       color = isEyeInWater == 0 ? renderTranslucentSpecular(color, data) : color;

  color1 = EncodeRGBE8(color);
}
#endif
