vec4 cubic(float x)
{
    float x2 = x * x;
    float x3 = x2 * x;
    vec4 w;
    w.x =   -x3 + 3*x2 - 3*x + 1;
    w.y =  3*x3 - 6*x2       + 4;
    w.z = -3*x3 + 3*x2 + 3*x + 1;
    w.w =  x3;
    return w / 6.f;
}

vec4 textureBilinear(sampler2D sampler, vec2 coord, ivec2 resolution){
  coord = coord * resolution - 0.5;

  ivec2 i = ivec2(floor(coord));
  vec2 f = coord - i;

  vec4 texel0 = texelFetch(sampler, (i + ivec2(0.0, 0.0)) % resolution, 0);
  vec4 texel1 = texelFetch(sampler, (i + ivec2(1.0, 0.0)) % resolution, 0);
  vec4 texel2 = texelFetch(sampler, (i + ivec2(0.0, 1.0)) % resolution, 0);
  vec4 texel3 = texelFetch(sampler, (i + ivec2(1.0, 1.0)) % resolution, 0);

  return mix(mix(texel0, texel1, f.x), mix(texel2, texel3, f.x), f.y);
}

vec4 bicubicTexture(sampler2D sampler, vec2 coord){
	ivec2 resolution = textureSize(sampler, 0);

  coord = coord * resolution - 0.5;

  vec2 f = fract(coord);

  coord -= f;

  vec2 ff = f * f;

  vec4 w0, w1;

  w0.xz = 1.0 -f;
  w0.xz *= w0.xz * w0.xz;

  w1.yw = ff *f;
  w1.xz = 3.0 * w1.yw + 4.0 - 6.0 * ff;

  w0.yw = 6.0 - w1.xz - w1.yw - w0.xz;

  vec4 s = w0 + w1;
  vec4 uv = coord.xxyy + vec4(-0.5, 1.5, -0.5, 1.5) / s;
       uv /= resolution.xxyy;

  vec4 texel0 = textureBilinear(sampler, uv.yw, resolution);
  vec4 texel1 = textureBilinear(sampler, uv.xw, resolution);
  vec4 texel2 = textureBilinear(sampler, uv.yz, resolution);
  vec4 texel3 = textureBilinear(sampler, uv.xz, resolution);

  vec2 m = s.xz / (s.xz + s.yw);

  return mix(mix(texel0, texel1, m.x), mix(texel2, texel3, m.x), m.y);
}
