float renderLambertDiffuse(vec3 n, vec3 l){
	return clamp01(dot(n, l)) * rPI;
}

float ExactCorrelatedG2(float alpha2, float NoV, float NoL) {
    float x = 2.0 * NoL * NoV;
    float y = (1.0 - alpha2);

    return x / (NoV * sqrt(alpha2 + y * (NoL * NoL)) + NoL * sqrt(alpha2 + y * (NoV * NoV)));
}

float GGX(float alpha2, float NoH) {
	float d = (NoH * alpha2 - NoH) * NoH + 1.0;

	return alpha2 / (d * d);
}

float SchlickFresnel(float f0, float f90, float LoH) {
    return (f90 - f0) * pow5(1. - LoH) + f0;
}

vec3 ExactFresnel(const vec3 n, const vec3 k, float c) {
    const vec3 k2= k * k;
	const vec3 n2k2 = n * n + k2;

    vec3 c2n = (c * 2.0) * n;
    vec3 c2 = vec3(c * c);

    vec3 rs_num = n2k2 - c2n + c2;
    vec3 rs_den = n2k2 + c2n + c2;

    vec3 rs = rs_num / rs_den;

    vec3 rp_num = n2k2 * c2 - c2n + 1.0;
    vec3 rp_den = n2k2 * c2 + c2n + 1.0;

    vec3 rp = rp_num / rp_den;

    return clamp01(0.5 * (rs + rp));
}

vec3 Fresnel(float f0, float f90, float LoH) {
  if(f0 > 0.34) {
		const vec3 diamondIOR = vec3(2.3757, 2.0847, 1.8453);
    const vec3 diamondK = vec3(4.2655, 3.7153, 3.1365);

		return ExactFresnel(diamondIOR, diamondK, LoH);
	} else if(f0 > 0.965) {
    const vec3 goldIOR = vec3(0.18299, 0.42108, 1.3734);
    const vec3 goldK = vec3(3.4242, 2.3459, 1.7704);

    return ExactFresnel(goldIOR, goldK, LoH);
  } else if(f0 > 0.45) {
    const vec3 ironIOR = vec3(2.9114, 2.9497, 2.5845);
    const vec3 ironK = vec3(3.0893, 2.9318, 2.7670);

    return ExactFresnel(ironIOR, ironK, LoH);
  } else {
    return vec3(SchlickFresnel(f0, f90, LoH));
  }
}

float renderSpecularGGX(vec3 normal, vec3 lightVector, vec3 viewVector, float f0, float roughness){
	vec3 H = normalize(lightVector - viewVector);

	float NdotL = clamp01(dot(normal, lightVector));
	float LdotH = clamp01(dot(H, lightVector));
	float NdotH = clamp01(dot(H, normal));
	float NdotV = clamp01(dot(normal, -viewVector));

	float denom = GGX(roughness, pow4(NdotH));

	float D = pow2(roughness) / (PI * pow2(denom));
	float F = f0 + (1.0 - f0) * pow5(1.0 - LdotH);

	float k2 = roughness / 2.0;
	float vis = pow2(GGX(k2, NdotH));

	return max0(NdotL * D * F * vis);
}

vec3 renderSpecularBRDF(vec3 normal, vec3 lightVector, vec3 viewVector, float f0, float alpha2){
	vec3 H = normalize(lightVector - viewVector);

	float NoL = clamp01(dot(normal, lightVector));
	float VoH = clamp01(dot(H, lightVector));
	float NoH = clamp01(dot(H, normal));
	float NoV = clamp01(dot(normal, -viewVector));

	return max0((Fresnel(f0, 1.0, VoH) * GGX(alpha2, NoH)) * ExactCorrelatedG2(alpha2, NoV, NoL) / 4.0) * NoL;
}

vec3 renderBlendMaterial(vec3 Kdiff, vec3 Kspec, vec3 diffuseColor, float f0) {
	if(f0 < 0.004) return Kdiff;

  float scRange = smoothstep(0.25, 0.45, f0);
  vec3  dielectric = Kspec + Kdiff;
  vec3  metal = diffuseColor * Kspec;

  return mix(dielectric, metal, scRange);
}
