#define diagonal2(m) vec2((m)[0].x, (m)[1].y)
#define diagonal3(m) vec3(diagonal2(m), m[2].z)
#define diagonal4(m) vec4(diagonal3(m), m[2].w)

#define transMAD(mat, v) (mat3(mat) * (v) + (mat)[3].xyz)
#define projMAD(mat, v) (diagonal3(mat) * (v) + (mat)[3].xyz)

const vec2 haltonSequenceOffsets[16] = vec2[16](
 	vec2(-1, -1),	vec2(0, -0.3333333), vec2(-0.5, 0.3333334), vec2(0.5, -0.7777778),
 	vec2(-0.75, -0.1111111), vec2(0.25, 0.5555556),	vec2(-0.25, -0.5555556), vec2(0.75, 0.1111112),
 	vec2(-0.875, 0.7777778), vec2(0.125, -0.9259259),	vec2(-0.375, -0.2592592),	vec2(0.625, 0.4074074),
 	vec2(-0.625, -0.7037037),	vec2(0.375, -0.03703701),	vec2(-0.125, 0.6296296), vec2(0.875, -0.4814815)
);

vec2 renderTemporalJitter() {
  #ifndef TEMPORAL_AA
  return vec2(0.0);
  #endif

  return (haltonSequenceOffsets[int(mod(frameCounter, 16))] * 2.0 - 1.0) * rPixel;
}

vec2 jitter = renderTemporalJitter();

vec3 renderViewPosition(vec2 coord, float depth){
  coord -= jitter * 0.5;
  vec3 view = vec3(coord, depth) * 2.0 - 1.0;
  return projMAD(gbufferProjectionInverse, view) / (view.z * gbufferProjectionInverse[2].w + gbufferProjectionInverse[3].w);
}

vec3 renderViewPosition(vec3 coord){
  vec3 view = coord * 2.0 - 1.0;
  return projMAD(gbufferProjectionInverse, view) / (view.z * gbufferProjectionInverse[2].w + gbufferProjectionInverse[3].w);
}

vec3 renderWorldPosition(vec3 view){
  return transMAD(gbufferModelViewInverse, view);
}

vec3 renderScreenPosition(vec3 view){
  return ((projMAD(gbufferProjection, view) / -view.z)) * 0.5 + 0.5 + vec3(jitter * 0.5, 0.0);
}

float renderShadowDistort(vec2 position){
  return length(position) * (length(position * 1.180) * SHADOW_DISTORTION + (1.0 - SHADOW_DISTORTION)) + 1.0;
}

vec2 renderShadowDistortPosition(vec2 position){
  return position / (length(position * 1.180) * SHADOW_DISTORTION + (1.0 - SHADOW_DISTORTION));
}

vec3 renderShadowDistortPosition(vec3 position){
  return vec3(renderShadowDistortPosition(position.xy), position.z * 0.35);
}
