#define SPEED 1.0	//[0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define TIME frameTimeCounter * SPEED

const float PI = 3.141592;
const float rPI = 1.0 / PI;
const float hPI = PI * 0.5;
const float TAU = PI * 2.0;
const float rTAU = 1.0 / TAU;
const float hTAU = TAU * 0.5;

const float PHI = sqrt(5.0) * 0.5 + 0.5;
const float LOG2 = log(2.0);
const float rLOG2	= 1.0 / LOG2;

const float goldenAngle = TAU / PHI / PHI;

float sumOf(vec2 x) { return x.x + x.y; }
#define clamp01(x) clamp(x, 0.0, 1.0)
#define max0(x) max(x, 0.0)
#define max1(x) max(x, 1.0)

#define fsign(x) (clamp01(x * 1e35) * 2.0 - 1.0)
#define fstep(x,y) clamp01((y - x) * 1e35)

#define lum0 vec3(0.2989, 0.5866, 0.1145)
#define lum1 vec3(0.2126, 0.7152, 0.0722)

#define curve(x) x * x * (3.0 - 2.0 * x)

#include "/program/utility/pow.glsl"
#include "/program/utility/noise.glsl"
#include "/program/utility/encode.glsl"
#include "/program/utility/position.glsl"
#include "/program/utility/bicubic.glsl"

#if defined deferred0 || defined deferred1 || defined composite0 || defined composite1
#include "/program/utility/diffuseOperator.glsl"
#endif

const float gamma = 2.4;

vec3 toGamma(vec3 x){
  //return pow(x, vec3(gamma));
  return mix(pow(x / 1.055 + (0.055 / 1.055), vec3(gamma)), x / 12.92, fstep(x, vec3(0.04045)));
}

vec3 toLinear(vec3 x){
  //return pow(x, vec3(1.0 / gamma));
  return mix(1.055 * pow(x, vec3(1.0 / gamma)) - 0.055, x * 12.92, fstep(x, vec3(0.0031308)));
}

vec3 waterScatterCoefficient = toGamma(vec3(50.0, 55.0, 60.0) / 255.0) * WATER_DENSITY;
vec3 waterAbsorptionCoefficient = toGamma(vec3(250.0, 150.0, 100.0) / 255.0) * (0.5 * WATER_DENSITY);
vec3 waterTransmittanceCoefficient = waterAbsorptionCoefficient + waterScatterCoefficient;

float renderLuminance601(vec3 x){
	return dot(x, lum0);
}

float renderLuminance701(vec3 x){
	return dot(x, lum1);
}

float linearDepth(float depth) {
  return (2.0 * near) / (far + near - depth * (far - near));
}

vec4 ToSH(float value, vec3 dir) {
  float transferl1 = 0.3849 * PI;
  float sqrt1OverPI = sqrt(rPI);
  float sqrt3OverPI = sqrt(rPI * 3.0);

  vec2 halfnhalf = vec2(0.5, -0.5);
  vec2 transfer = vec2(PI * sqrt1OverPI, transferl1 * sqrt3OverPI);

  vec4 foo = halfnhalf.xyxy * transfer.xyyy;

  return foo * vec4(1.0, dir.yzx) * value;
}

vec3 FromSH(vec4 cR, vec4 cG, vec4 cB, vec3 lightDir) {
  float sqrt1OverPI = sqrt(rPI);
  float sqrt3OverPI = sqrt(3.0 * rPI);

  vec2 halfnhalf = vec2(0.5, -0.5);
  vec2 sqrtOverPI = vec2(sqrt1OverPI, sqrt3OverPI);
  vec4 foo = halfnhalf.xyxy * sqrtOverPI.xyyy;

  vec4 sh = foo * vec4(1.0, lightDir.yzx);

  return vec3(dot(sh,cR),
        		  dot(sh,cG),
        			dot(sh,cB));
}

vec3 blackbody(float t){
    // http://en.wikipedia.org/wiki/Planckian_locus

    vec4 vx = vec4(-0.2661239e9,-0.2343580e6,0.8776956e3,0.179910);
    vec4 vy = vec4(-1.1063814,-1.34811020,2.18555832,-0.20219683);
    float it = 1./t;
    float it2= it*it;
    float x = dot(vx,vec4(it*it2,it2,it,1.));
    float x2 = x*x;
    float y = dot(vy,vec4(x*x2,x2,x,1.));
    float z = 1. - x - y;

    // http://www.brucelindbloom.com/index.html?Eqn_RGB_XYZ_Matrix.html
    mat3 xyzToSrgb = mat3(
         3.2404542,-1.5371385,-0.4985314,
        -0.9692660, 1.8760108, 0.0415560,
         0.0556434,-0.2040259, 1.0572252
    );

    vec3 srgb = vec3(x/y,1.,z/y) * xyzToSrgb;
    return max(srgb,0.);
}

vec2 sincos(float x){
    return vec2(sin(x), cos(x));
}

vec2 circlemap(float i, float n){
	return sincos(i * n * goldenAngle) * sqrt(i);
}

vec3 circlemapL(float i, float n){
	return vec3(sincos(i * n * goldenAngle), sqrt(i));
}

vec2 rotate(vec2 vector, float r) {
    float c = cos(r), s = sin(r);
    return vector * mat2(c, -s, s, c);
}

vec3 renderProjectSky(vec2 coord) {
  coord *= vec2(TAU, PI);
	vec2 lon = sincos(coord.x) * sin(coord.y);
	return vec3(lon.x, 2.0 / PI * coord.y - 1.0, lon.y);
}

vec2 renderUnprojectSky(vec3 dir) {
  float lonlat = atan(-dir.x, -dir.z);
  return vec2(lonlat * rTAU + 0.5, 0.5 * dir.y + 0.5);
}

vec3 renderDesaturation(vec3 color) {
  return mix(color, renderLuminance601(color) * vec3(0.2, 0.4, 0.7), vec3(0.9));
}

mat3 renderRotMat(vec3 x,vec3 y){
   float d = dot(x,y);
   vec3 cr = cross(y,x);

   float s = length(cr);

   float id = 1.-d;

   vec3 m = cr/s;

   vec3 m2 = m*m*id+d;
   vec3 sm = s*m;

   vec3 w = (m.xy*id).xxy*m.yzz;

   return mat3(
	   m2.x,     w.x-sm.z, w.y+sm.y,
	   w.x+sm.z, m2.y,     w.z-sm.x,
	   w.y-sm.y, w.z+sm.x, m2.z
   );
}
